"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.FiredevCrudDeamon = void 0;
var tslib_1 = require("tslib");
//#region imports
//#region @backend
var background_worker_process_1 = require("background-worker-process");
//#endregion
//#region isomorphic
var firedev_crud_1 = require("firedev-crud");
var tnp_helpers_1 = require("tnp-helpers");
var typescript_class_helpers_1 = require("typescript-class-helpers");
var firedev_ports_1 = require("firedev-ports");
//#endregion
var daemon_entity_1 = require("./daemon-entity");
var deamon_controller_1 = require("./deamon-controller");
var FiredevCrudDeamon = exports.FiredevCrudDeamon = /** @class */ (function (_super) {
    tslib_1.__extends(FiredevCrudDeamon, _super);
    //#endregion
    //#region constructor
    function FiredevCrudDeamon(controllers, entities) {
        if (controllers === void 0) { controllers = []; }
        if (entities === void 0) { entities = []; }
        var _this = _super.call(this, controllers, entities) || this;
        _this.controllers = controllers;
        _this.entities = entities;
        //#region fields & getters
        _this.watchers = {};
        //#region @backend
        controllers.push(firedev_ports_1.PortsController);
        entities.push(firedev_ports_1.PortInstance);
        _this.entities = tnp_helpers_1.Helpers.arrays.uniqArray(entities);
        _this.controllers = tnp_helpers_1.Helpers.arrays.uniqArray(controllers);
        return _this;
        //#endregion
    }
    //#endregion
    //#region api
    //#region  api / ports manager
    FiredevCrudDeamon.prototype.getPortsManager = function () {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var portsManager;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.getCtrlInstanceBy(firedev_ports_1.PortsController).manager];
                    case 1: return [4 /*yield*/, (_a.sent())];
                    case 2:
                        portsManager = _a.sent();
                        return [2 /*return*/, portsManager];
                }
            });
        });
    };
    //#endregion
    //#region api / init
    FiredevCrudDeamon.prototype.init = function (options) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0: 
                    //#region @backend
                    return [4 /*yield*/, _super.prototype.init.call(this, options)];
                    case 1:
                        //#region @backend
                        _a.sent();
                        if (!global.useWorker) return [3 /*break*/, 4];
                        return [4 /*yield*/, this.initDeamon(options.recreate || global.restartWorker)];
                    case 2:
                        _a.sent();
                        return [4 /*yield*/, this.initWatchingDb()];
                    case 3:
                        _a.sent();
                        _a.label = 4;
                    case 4: return [2 /*return*/];
                }
            });
        });
    };
    //#endregion
    //#region api / create instance
    FiredevCrudDeamon.prototype.createInstance = function (classFN, entities, registerdOnPort, startNew) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var res;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, background_worker_process_1.WorkersFactor.create(classFN, entities, registerdOnPort, {
                            killAlreadRegisteredProcess: startNew,
                            startWorkerServiceAsChildProcess: startNew,
                            disabledRealtime: false,
                            preventSameContexts: true,
                        })];
                    case 1:
                        res = _a.sent();
                        if (process.platform === 'win32' && startNew) {
                            tnp_helpers_1.Helpers.info('Waiting 10 seconds on windows platofrom...');
                            tnp_helpers_1.Helpers.sleep(10);
                        }
                        return [2 /*return*/, res];
                }
            });
        });
    };
    //#endregion
    //#region api / init watching db.json
    FiredevCrudDeamon.prototype.initWatchingDb = function () {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            return tslib_1.__generator(this, function (_a) {
                return [2 /*return*/];
            });
        });
    };
    //#endregion
    //#region api / init deamon
    FiredevCrudDeamon.prototype.initDeamon = function (recreate) {
        if (recreate === void 0) { recreate = false; }
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var entities, portsManager;
            var _this = this;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        entities = [daemon_entity_1.DbUpdateProjectEntity];
                        return [4 /*yield*/, this.getCtrlInstanceBy(firedev_ports_1.PortsController)];
                    case 1: return [4 /*yield*/, (_a.sent()).manager];
                    case 2:
                        portsManager = _a.sent();
                        return [4 /*yield*/, portsManager.registerOnFreePort({
                                name: typescript_class_helpers_1.CLASS.getName(deamon_controller_1.DbDaemonController)
                            }, {
                                actionWhenAssignedPort: function (itWasRegisterd, registerdOnPort) { return tslib_1.__awaiter(_this, void 0, void 0, function () {
                                    var res, isHealtyWorker, copyDataToWorker, allData;
                                    return tslib_1.__generator(this, function (_a) {
                                        switch (_a.label) {
                                            case 0:
                                                tnp_helpers_1.Helpers.log("[tnp-db][deamon] ".concat(itWasRegisterd ? 'already' : 'inited', " on port: ").concat(registerdOnPort));
                                                return [4 /*yield*/, this.createInstance(deamon_controller_1.DbDaemonController, entities, registerdOnPort, (!itWasRegisterd || recreate))];
                                            case 1:
                                                res = _a.sent();
                                                return [4 /*yield*/, res.instance.$$healty];
                                            case 2:
                                                isHealtyWorker = _a.sent();
                                                copyDataToWorker = (!itWasRegisterd || recreate || !isHealtyWorker);
                                                if (!!isHealtyWorker) return [3 /*break*/, 4];
                                                res.context.destroy();
                                                return [4 /*yield*/, this.createInstance(deamon_controller_1.DbDaemonController, entities, registerdOnPort, true)];
                                            case 3:
                                                res = _a.sent();
                                                _a.label = 4;
                                            case 4:
                                                if (!copyDataToWorker) return [3 /*break*/, 6];
                                                allData = tnp_helpers_1.Helpers.readJson(this.location);
                                                return [4 /*yield*/, res.instance.copyAllToWorker(allData, this.location).received];
                                            case 5:
                                                _a.sent();
                                                _a.label = 6;
                                            case 6:
                                                this.context = res.context;
                                                this.worker = res.instance;
                                                return [2 /*return*/];
                                        }
                                    });
                                }); }
                            })];
                    case 3:
                        _a.sent();
                        return [2 /*return*/];
                }
            });
        });
    };
    //#endregion
    //#region api / get worker port
    FiredevCrudDeamon.prototype.getWokerPort = function () {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var portsManager;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.getCtrlInstanceBy(firedev_ports_1.PortsController)];
                    case 1: return [4 /*yield*/, (_a.sent()).manager];
                    case 2:
                        portsManager = _a.sent();
                        return [4 /*yield*/, portsManager.getPortOf({ name: typescript_class_helpers_1.CLASS.getName(deamon_controller_1.DbDaemonController) })];
                    case 3: return [2 /*return*/, _a.sent()];
                }
            });
        });
    };
    //#endregion
    //#region api / kill worker
    FiredevCrudDeamon.prototype.killWorker = function () {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var portsManager, error_1, portTokill;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.getPortsManager()];
                    case 1:
                        portsManager = _a.sent();
                        tnp_helpers_1.Helpers.log("[killing worker] starting killing db worker...");
                        _a.label = 2;
                    case 2:
                        _a.trys.push([2, 4, , 5]);
                        return [4 /*yield*/, this.worker.triggerSave().received];
                    case 3:
                        _a.sent();
                        tnp_helpers_1.Helpers.log("[killing worker] trigerr save OK");
                        return [3 /*break*/, 5];
                    case 4:
                        error_1 = _a.sent();
                        tnp_helpers_1.Helpers.log("[killing worker] trigerr save ERROR");
                        return [3 /*break*/, 5];
                    case 5: return [4 /*yield*/, portsManager.getPortOf({ name: typescript_class_helpers_1.CLASS.getName(deamon_controller_1.DbDaemonController) })];
                    case 6:
                        portTokill = _a.sent();
                        return [4 /*yield*/, tnp_helpers_1.Helpers.killProcessByPort(portTokill)];
                    case 7:
                        _a.sent();
                        return [2 /*return*/];
                }
            });
        });
    };
    FiredevCrudDeamon = tslib_1.__decorate([
        typescript_class_helpers_1.CLASS.NAME('FiredevCrudDeamon'),
        tslib_1.__metadata("design:paramtypes", [Array, Array])
    ], FiredevCrudDeamon);
    return FiredevCrudDeamon;
}(firedev_crud_1.FiredevCrud));
//# sourceMappingURL=firedev-crud-deamon.js.map